#include <stdio.h>
#include <stdlib.h>
#include <stdbool.h>
#include <stdint.h>
#ifdef WIN32
#include <windows.h>
#else
#include <unistd.h>
#include <signal.h>
#endif // WIN32
#include "udp_marvelmind.h"

bool terminateProgram=false;

#ifdef WIN32
BOOL CtrlHandler( DWORD fdwCtrlType )
{
    if ((fdwCtrlType==CTRL_C_EVENT ||
            fdwCtrlType==CTRL_BREAK_EVENT ||
            fdwCtrlType==CTRL_CLOSE_EVENT ||
            fdwCtrlType==CTRL_LOGOFF_EVENT ||
            fdwCtrlType==CTRL_SHUTDOWN_EVENT) &&
            (terminateProgram==false))
    {
        terminateProgram=true;
        return true;
    }
    else return false;
}
#else
void CtrlHandler(int signum)
{
    terminateProgram=true;
}
#endif

#ifdef WIN32
void sleep(unsigned int seconds)
{
    Sleep (seconds*1000);
}
#endif

int main (int argc, char *argv[])
{
    uint8_t beaconRequestAddress;
    if (argc>=2) beaconRequestAddress=atoi(argv[1]);
    else beaconRequestAddress=0;

    // get server address from command line
    const char * serverAddress;
    if (argc>=3) serverAddress=argv[2];
    else serverAddress=DEFAULT_UDP_SERVER_ADDRESS;

    uint16_t serverPort;
    if (argc>=4) serverPort=atoi(argv[3]);
    else serverPort=DEFAULT_UDP_SERVER_PORT;

    // Init
    struct MarvelmindUDP * udp=createMarvelmindUDP ();
    if (udp==NULL)
    {
        puts ("Error: Unable to create MarvelmindUDP");
        return -1;
    }
    udp->serverAddress=serverAddress;
    udp->serverPort= serverPort;
    udp->beaconRequestAddress= beaconRequestAddress;
    udp->requestRateHz= 16;
    udp->verbose=true; // show errors and warnings
    startMarvelmindUDP (udp);

    // Set Ctrl-C handler
#ifdef WIN32
    SetConsoleCtrlHandler( (PHANDLER_ROUTINE) CtrlHandler, TRUE );
#else
    signal (SIGINT, CtrlHandler);
    signal (SIGQUIT, CtrlHandler);
#endif

    // Main loop
    while ((!terminateProgram) && (!udp->terminationRequired))
    {
        //sleep (3);
        printPositionFromMarvelmindUDP (udp, true);
    }

    // Exit
    stopMarvelmindUDP (udp);
    destroyMarvelmindUDP (udp);
    return 0;
}
